@@ -2,6 +2,7 @@
 '''stuf utilities'''
 
 from inspect import ismodule
+from itertools import starmap
 try:
     from collections import OrderedDict
 except  ImportError:
@@ -59,15 +60,61 @@ def deepget(this, key, default=None):
         return default
 
 
-def exhaust(iterable, exception=StopIteration, _next=next):
+def exhaust(iterable, exception=StopIteration, _n=next):
     '''
     call next on an iterator until it's exhausted
 
     @param iterable: an iterable to exhaust
+    @param exception: exception that marks end of iteration
     '''
     try:
         while True:
-            next(iterable)
+            _n(iterable)
+    except exception:
+        pass
+
+
+def breakcount(func, length):
+    '''
+    run an iterator until it reaches its original length
+
+    @param iterable: an iterable to exhaust
+    '''
+    while length:
+        yield func()
+        length -= 1
+
+
+def exhaustmap(mapping, call, filter=False, exception=StopIteration, _n=next):
+    '''
+    call `next` on an iterator until it's exhausted
+
+    @param mapping: a mapping to exhaust
+    @param call: call to handle what survives the filter
+    @param filter: a filter to apply to mapping (default: `None`)
+    @param exception: exception sentinel (default: `StopIteration`)
+    '''
+    subiter = ifilter(filter, items(mapping)) if filter else items(mapping)
+    iterable = starmap(call, subiter)
+    try:
+        while True:
+            _n(iterable)
+    except exception:
+        pass
+
+
+def exhaustcall(call, iterable, exception=StopIteration, _n=next):
+    '''
+    call function on an iterator until it's exhausted
+
+    @param call: call that does the exhausting
+    @param iterable: iterable to exhaust
+    @param exception: exception marking end of iteration
+    '''
+    iterable = map(call, iterable)
+    try:
+        while True:
+            _n(iterable)
     except exception:
         pass
 
@@ -152,16 +199,20 @@ def inverse_lookup(value, this, default=None):
         return default
 
 
-def iterexcept(func, exception):
+def iterexcept(func, exception, start=None):
     '''
     call a function repeatedly until an exception is raised
 
     Converts a call-until-exception interface to an iterator interface. Like
     `__builtin__.iter(func, sentinel)` but uses an exception instead of a
     sentinel to end the loop.
+
+    Raymond Hettinger Python Cookbook recipe # 577155
     '''
     try:
-        while True:
+        if start is not None:
+            yield start()
+        while 1:
             yield func()
     except exception:
         pass
